# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
#  script:  reconfig_raid0-1.sh 
#
# purpose:  converts decoder head unit jbod,
#           scsi raid0 virtual drives to
#           raid1 per a reliability design
#           change
#          
# requires: root privileges, free space in
#           /var/netwitness mounted logical
#           volume
#
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
#
#!/bin/bash
#
shopt -sq extglob
scriptver="2016.09.07.0"
mylog=./reconfig_raid0-1.log
myruntime=`date`
echo "----------------- ${myruntime} ---------------" >> ${mylog}
#
# Print usage info
function usage () {

	echo -e "Reconfigure decodersmall from RAID0 to RAID1"
	echo -e "Usage: $0 [-f|-p] [Username  Password]"
	echo -e "\nNOTE:  If Password contains certain special characters like @ must be placed within 'single quotes'."
	echo -e "         If your password doesn't work, please single quote it."
	echo
	echo -e "-f option will force all conversions to be automatic"
	echo -e "   If this is the only option, script will use the default credentials(admin/netwitness)"
	echo
	echo -e "-p option will prompt for username, password and conversion options"
	echo
	echo -e "Examples:"
	echo -e "./reconfig-raid0-1.sh                            (provide usage information)"
	echo -e "./reconfig-raid0-1.sh -p                         (prompt all questions)"
	echo -e "./reconfig-raid0-1.sh -f                         (force conversion, default credentials)"
	echo -e "./reconfig-raid0-1.sh -p admin 'N3tW1tn3$$'   (prompt for conversion, use entered credentials)"
	echo -e "./reconfig-raid0-1.sh -f admin 'N3tW1tn3$$'   (force conversion, use entered credentials)"
	echo
	exit 1
}
#
## Script with correct # args?
if [ $# -eq 0 ]; then
	usage 
elif [ $# -eq 1 ]; then
	if [ $1 == "-p" ]; then 
		FORCE=0
	elif [ $1 == "-f" ]; then 
		FORCE=1
	else
		usage
	fi
elif [ $# -eq 3 ]; then
	if [ $1 == "-p" ]; then
		FORCE=0
	elif [ $1 == "-f" ]; then
		FORCE=1
	else
		usage
	fi
else
	usage
fi
#
function checkcreds {
	curl -s -k -u $userName:$passWord http://localhost:50104/users/accounts/?force-content-type=text/plain | grep -q $userName
	if [ $? -eq 0 ]; then
		serVice='http'
		gotGood='1'
		return
	fi
	curl -s -k -u $userName:$passWord https://localhost:50104/users/accounts/?force-content-type=text/plain | grep -q $userName
	if [ $? -eq 0 ]; then
		serVice='https'
		gotGood='1'
		return
	fi
	echo "NetWitness Service login failed.  Please enter valid login credentials." | tee -a ${mylog}
	exit 1
}
#
function getR0Count {
raidCount=`/opt/MegaRAID/MegaCli/MegaCli64 -ldinfo -Lall -aALL | grep RAID | grep -c "Primary-0"`
echo "raidCount = ${raidCount}" >> ${mylog} 
}
#
function stopServices {
echo "Stopping Capture" | tee -a ${mylog}
curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder?msg=stop&force-content-type=text/plain&expiry=600"
echo "Disabling Capture Autostart" | tee -a ${mylog}
curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=off"
echo "Allowing time for Concentrator to complete aggregation" | tee -a ${mylog}
for i in {0..30..1};do echo -n "." && sleep 1;done && echo "Done"
echo "Shutting Down Decoder Service" | tee -a ${mylog}
stop nwdecoder
echo "Waiting for Decoder service to complete shutdown" | tee -a ${mylog}
for i in {0..30..1};do echo -n "." && sleep 1;done && echo "Done"
}
#
function raidInfo {
RAID_CONFIG=`mktemp`
nwraidutil.pl | egrep H8[321]0 -m1 &> ${RAID_CONFIG}
nwraidutil.pl | egrep 'Raid Level 0' -m1 -A2 &>> ${RAID_CONFIG}
ADPTR=`grep Adapter ${RAID_CONFIG} | awk '{print $2}'`
VD=`grep Virtual ${RAID_CONFIG} | awk '{print $3}'`
echo "line ${LINENO}: VD = ${VD}" >> ${mylog}
ENCL=`grep -m1 Enclosure ${RAID_CONFIG} | awk '{print $4}'`
SLT0=`grep Slot ${RAID_CONFIG} | awk '{print $6}' | head -1`
SLT1=`grep Slot ${RAID_CONFIG} | awk '{print $6}' | tail -1`
SCSI=`cat /proc/scsi/scsi | grep -e  H8[321]0 -m1 -B1 | head -1 | awk '{print $2}' | awk -Fi '{print $2}'`
PV=`echo "/dev/"``ls /sys/class/scsi_device/$SCSI\:*\:$VD\:*/device/block`
echo "line ${LINENO}: PV = ${PV}" >> ${mylog}
VG=`pvs | grep $PV | awk '{print $2}'`
echo "line ${LINENO}: VG = ${VG}" >> ${mylog}
LVS=`lvs | grep -w $VG | awk '{print $1}'`
echo "line ${LINENO}: LVS = ${LVS}" >> ${mylog}
LVC=`echo $LVS | wc -w`
echo "line ${LINENO}: LVCount = ${LVC}" >> ${mylog}
VGN=`echo -n $VG | tail -c 1`; if [ $VGN == "l" ];then VGN="";fi
echo "line ${LINENO}: VGN = ${VGN}" >> ${mylog}
echo 
echo "Found RAID 0 Configuration:" | tee -a ${mylog}
echo 
cat $RAID_CONFIG | tee -a ${mylog}
echo
}
#
function getBackup {

local bkupsizek
local targetbsize
local targetfreeb
local targetfreek
local usrinput

echo "Determining last Metadb and Sessiondb files" | tee -a ${mylog}
lastMetaFile=`find /var/netwitness/decoder/ -name metadb -exec find {} -name *.nwmdb \; -print | awk -F- '{print "meta-"$2}' | sort | tail -1`
echo "lastMetaFile = ${lastMetaFile}" >> ${mylog}
lastSessFile=`find /var/netwitness/decoder/ -name sessiondb -exec find {} -name *.nwsdb \; -print | awk -F- '{print "session-"$2}' | sort | tail -1`
echo "lastSessFile = ${lastSessFile}" >> ${mylog}
METAFILE=$(find /var/netwitness/decoder/ -name `echo $lastMetaFile` -print)
echo "METAFILE = ${METAFILE}" >> ${mylog}
SESSIONFILE=$(find /var/netwitness/decoder/ -name `echo $lastSessFile` -print)
echo "SESSIONFILE = ${SESSIONFILE}" >> ${mylog}
if [ -z ${METAFILE} ] || [ -z ${SESSIONFILE} ]; then
	echo
	echo 'Missing decoder capture data detected, assuming this is a new system'
	read -p "Continuing may cause loss of data, proceed without backup (y/Y)? " usrinput
	if [ ${usrinput} = "y" ] || [ ${usrinput} = "Y" ]; then 
		nobackup='true'
		return
	else 
		exitonerror "${LINENO}" 'missing decoder data detected, user canceled raid conversion' 'curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1
	fi
	
fi

# check for available free space in /var/netwitness
let bkupsizek=`du -skc ${METAFILE} ${METAFILE}index ${METAFILE}.manifest ${SESSIONFILE} ${SESSIONFILE}.manifest /var/netwitness/decoder/index/managed-values-* | grep -E '^[0-9]+[[:space:]]+total$' | awk '{print $1}'`
echo 'checking size of backup and target folder /var/netwitness free space' >> ${mylog}
echo "line ${LINENO}: bkupsizek = ${bkupsizek}" >> ${mylog}
targetbsize=`stat -f -c %S /var/netwitness`
targetfreeb=`stat -f -c %f /var/netwitness`
let targetfreek=`expr ${targetbsize} \* ${targetfreeb} / 1024`
echo "line ${LINENO}: targetfreekb = ${targetfreek}" >> ${mylog}
if [ ${bkupsizek} -gt ${targetfreek} ]; then
	echo "line ${LINENO}: backup size ${bkupsizek}kb greater than /var/netwitness target free space ${targetfreek}kb" | tee -a ${mylog}
	echo "unable to complete backup, the script will exit now " | tee -a ${mylog}
	curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"
	start nwdecoder
	exit 1	
fi
echo "Backing up Index files, Last SessionDB Files, and Last MetaDB files" | tee -a ${mylog}
if [ -d /var/netwitness/decoderbkup ]; then
	exitonerror "${LINENO}" 'found previous backup attempt folder /var/netwitness/decoderbkup, removing' 'rm -rf /var/netwitness/decoderbkup'
fi
mkdir /var/netwitness/decoderbkup || exitonerror "${LINENO}" 'error creating backup folder /var/netwitness/decoderbkup' 'curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1
cp -Rp /var/netwitness/decoder/index/managed-values-* /var/netwitness/decoderbkup/ || exitonerror "${LINENO}" 'error backing up decoder index files' 'curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1
cp -p `echo $METAFILE` /var/netwitness/decoderbkup || exitonerror "${LINENO}" "error backing up ${METAFILE}" 'curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1
cp -p `echo -n $METAFILE`index /var/netwitness/decoderbkup || exitonerror "${LINENO}" "error backing up ${METAFILE}index" 'curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1
cp -p `echo -n $METAFILE`.manifest /var/netwitness/decoderbkup || exitonerror "${LINENO}" "error backing up ${METAFILE}.manifest" 'curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1
cp -p `echo $SESSIONFILE` /var/netwitness/decoderbkup || exitonerror "${LINENO}" "error backing up ${SESSIONFILE}" 'curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1
cp -p `echo $SESSIONFILE`.manifest /var/netwitness/decoderbkup || exitonerror "${LINENO}" "error backing up ${SESSIONFILE}.manifest" 'curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1

}
function fixRaid  {
echo "Unmounting Filesystems" | tee -a ${mylog}
if [ -z $VGN ];then
	MOUNTS=`ls /var/netwitness/decoder/|awk '/(index|meta|session|packet)/'`
	for m in `echo $MOUNTS`
	do
		umount /var/netwitness/decoder/$m || exitonerror "${LINENO}" "error unmounting /var/netwitness/decoder/${m}" '/bin/mount -a; curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1
	done
	umount /var/netwitness/decoder || exitonerror "${LINENO}" 'error unmounting /var/netwitness/decoder' '/bin/mount -a; curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1
else
	MOUNTS=`ls /var/netwitness/decoder/|awk '/(metadb'$VGN'|sessiondb'$VGN'|packetdb'$VGN')/'`
	for m in `echo $MOUNTS`
	do
		umount /var/netwitness/decoder/$m || exitonerror "${LINENO}" "error unmounting /var/netwitness/decoder/${m}" '/bin/mount -a; curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"; start nwdecoder' 1
	done
fi
#
echo "Removing '${LVS}' Logical Volumes" | tee -a ${mylog}
for LV in `echo $LVS`
do
	echo "removing logical volume /dev/$VG/$LV" | tee -a ${mylog}
	lvremove -f /dev/$VG/$LV
done
echo "Removing '${VG}' Volume Group" | tee -a ${mylog}
vgremove -f $VG
echo "Removing '${PV}' Physical Volume" | tee -a ${mylog}
pvremove -f $PV
echo "Removing RAID0 Virtual Drive '${VD}'" | tee -a ${mylog}
/opt/MegaRAID/MegaCli/MegaCli64 -cfglddel -L$VD -a$ADPTR
echo "Creating RADI1 Virtual Drive $VD" | tee -a ${mylog}
/opt/MegaRAID/MegaCli/MegaCli64 -cfgldadd -R1[$ENCL:$SLT0,$ENCL:$SLT1] -WB -ADRA -Cached -CachedBadBBU -strpsz128 -a$ADPTR
echo "Creating '${PV}' Physical Volume" | tee -a ${mylog}
pvcreate $PV
echo "Creating '${VG}' Volume Group" | tee -a ${mylog}
vgcreate $VG $PV
echo "Creating Logical Volumes, Filesystems, and Mounts" | tee -a ${mylog}
if [ -z $VGN ];then
	lvcreate -L 30G $VG -n decoroot
	mkfs.xfs /dev/$VG/decoroot
	mount /var/netwitness/decoder
	for m in `echo $MOUNTS`;do mkdir /var/netwitness/decoder/$m;done
	lvcreate -L 30G $VG -n index
	mkfs.xfs /dev/$VG/index
	mount /var/netwitness/decoder/index
fi	
lvcreate -l 10%FREE $VG -n sessiondb
mkfs.xfs /dev/$VG/sessiondb
mount /var/netwitness/decoder/sessiondb$VGN
if [ -n $VGN ];then mkdir /var/netwitness/decoder/sessiondb$VGN/sessiondb;fi
lvcreate -l 100%FREE $VG -n metadb
mkfs.xfs /dev/$VG/metadb
mount /var/netwitness/decoder/metadb$VGN
if [ -n $VGN ];then mkdir /var/netwitness/decoder/metadb$VGN/metadb;fi
mount -a
}
#
function putBackup {
echo "Restoring Index Files" | tee -a ${mylog}
cp -Rp /var/netwitness/decoderbkup/managed-values-* /var/netwitness/decoder/index/
echo "Restoring Metadb Files" | tee -a ${mylog}
cp -p /var/netwitness/decoderbkup/meta* /var/netwitness/decoder/metadb/
echo "Restoring Sessiondb Files" | tee -a ${mylog}
cp -p /var/netwitness/decoderbkup/session* /var/netwitness/decoder/sessiondb/
}
#
function finishUp {
echo "Starting Decoder Service" | tee -a ${mylog}
start nwdecoder
echo "Allowing Decoder Service to complete startup" | tee -a ${mylog}
echo "Running Reconfig in 120 seconds..." | tee -a ${mylog}
for i in {0..120..5};do echo -n "." && sleep 5;done && echo "."
echo "Reconfig MetaDB and SessionDB Database roll-off size" | tee -a ${mylog}
curl -s -k -u $userName:$passWord "$serVice://localhost:50104/database?msg=reconfig&force-content-type=text/plain&expiry=600&update=1"
echo "Reconfig Index storage roll-off size" | tee -a ${mylog}
curl -s -k -u $userName:$passWord "$serVice://localhost:50104/index?msg=reconfig&force-content-type=text/plain&expiry=600&update=1"
echo "Enable Capture Autostart"
curl -s -k -u $userName:$passWord "$serVice://localhost:50104/decoder/config/capture.autostart?msg=set&force-content-type=text/plain&expiry=600&value=on"
echo "Restarting Decoder Service to activiate size changes" | tee -a ${mylog}
restart nwdecoder
echo "Removing backup files" | tee -a ${mylog}
rm -rf /var/netwitness/decoderbkup
rm -f /tmp/tmp*
}
#
function exitonerror {
	local linenum
	local message
	local action
	local exitcode
	local count
	# get parameters
	let count=1
	while ! [ -z "${1}" ]
	do
		if [ ${count} -eq 1 ]; then
			linenum="${1}"
		elif [ ${count} -eq 2 ]; then
			message="${1}"
		elif [ ${count} -eq 3 ]; then
			action="${1}"	
		else
			exitcode="${1}"
		fi
		shift
		let count=${count}+1
	done
	# require at least line number and message
	if [ ${count} -lt 3 ]; then
		return
	fi
	
	if ! [ -z "${exitcode}" ]; then
		echo "line ${LINENO}: ${message}" | tee -a ${mylog}
		echo "crtical error, the scipt will exit now" | tee -a ${mylog}
	else
		echo "line ${LINENO}: ${message}" | tee -a ${mylog}
		echo "non critical error, the scipt will continue" | tee -a ${mylog}
		sleep 2 
	fi
	# take corrective acton, prepend script call strings with 'script:'
	if ! [ -z "${action}" ] && [[ `echo "${action}" | grep -E '[[:alnum:]]+'` ]]; then
		if [[ `echo "${action}" | grep '^script:'` ]];then
			action=${action#script:}
			${action}
		else		
			/bin/bash -c "${action}"
		fi
	fi
	# exit on error	
	if ! [ -z "${exitcode}" ]; then
		if [[ "${exitcode}" = '0' || "${exitcode}" = '1' ]]; then
			exit ${exitcode}
		else
			exit 1
		fi
	else
		return
	fi 
}

# Main Program
#
# check for root uid
if ! [[ `whoami | grep '^root$'` ]]; then
	echo 'script must be run as root user, exiting' | tee -a ${mylog}
	exit 1
fi

# only run on decoder head units
if ! [[ `rpm -q 'nwdecoder' | grep -E 'nwdecoder-[0-9]+\.[0-9]+'` ]] || [[ `rpm -q 'nwconcentrator' | grep -E 'nwconcentrator-[0-9]+\.[0-9]+'` ]]; then
	echo 'script should only be run on decoder appliances, exiting' | tee -a ${mylog}
	exit 1
fi

# check for decodersmall[0-9]* volume groups
if ! [[ `vgs --all -o vg_name | grep 'decodersmall'` ]]; then
	echo 'no affected volume groups were detected' | tee -a ${mylog}
	echo 'nothing to do, the script will exit now' | tee -a ${mylog}
	exit 0
fi

# check for running decoder service, attempt to start if stopped
if ! [[ `status nwdecoder | grep 'run'` ]]; then
	echo 'nwdecoder not running, starting service' | tee -a ${mylog}
	start nwdecoder
	echo "Allowing Decoder Service to complete startup" | tee -a ${mylog}
	echo "Continuing in 60 seconds..." | tee -a ${mylog}
	for i in {0..60..5};do echo -n "." && sleep 5;done && echo "."
	if ! [[ `status nwdecoder | grep 'run'` ]]; then
		exitonerror ${LINENO} 'nwdecoder service failed to start' ' ' 1	
	fi
fi

unset userName
unset passWord
gotGood='0'
while [ $gotGood -eq 0 ]; do 
	if [ $# -eq 1 ] && [ $1 == "-p" ]; then
		read -p "NW Administrator username (admin): " userName
		if [ -z $userName ]; then userName='admin'; fi
		read -s -p "NW Administrator password (netwitness): " passWord 
		echo $'\n'
		if [ -z $passWord ]; then passWord='netwitness'; fi
	elif [ $# -eq 1 ] && [ $1 == "-f" ]; then
		userName='admin'
		passWord='netwitness'
	elif [ $# -eq 3 ]; then
		userName=$2
		passWord=$3
	else
		usage
	fi
	checkcreds
done
getR0Count
if [ "${raidCount}" = '0' ]; then
	echo 'no scsi raid0 virtual drives were found' | tee -a ${mylog}
	echo 'nothing to do, the script will exit now' | tee -a ${mylog}
	exit 0
fi
# prompt for raid conversion
if [ $FORCE -eq 0 ]; then
	read -p "Do you want to convert RAID0 disk arrays to RAID1? (y/n) " conVert
	if [ $conVert = "y" ] || [ $conVert = "Y" ]; then 
		echo -n 
	else 
		echo 'raid conversion canceled, script will exit now'
		exit 0		
	fi
fi
stopServices
getBackup
until [ "$raidCount" -eq 0 ]
do
	raidInfo
	echo "Reconfiguring RAID0 drives on Adapter: "$ADPTR", Enclosure: "$ENCL", Slots: "$SLT0" & "$SLT1 "."	
	fixRaid
	getR0Count
	sleep 2
done
if ! [ ${nobackup} ]; then
	putBackup
fi
finishUp
echo "RAID0 to RAID1 Reconfiguration Complete, Please verify Capture has started" | tee -a ${mylog}
